/**********************************************************************
 * $Id:: lcd_colorbars.c 1260 2008-10-28 23:20:43Z wellsk             $
 *
 * Project: NXP IRD_2.0 2478 LCD example
 *
 * Description:
 *     Draws color bars on the LCD. Requires RGB565 mode.
 *
 **********************************************************************
 * Software that is described herein is for illustrative purposes only  
 * which provides customers with programming information regarding the  
 * products. This software is supplied "AS IS" without any warranties.  
 * NXP Semiconductors assumes no responsibility or liability for the 
 * use of the software, conveys no license or title under any patent, 
 * copyright, or mask work right to the product. NXP Semiconductors 
 * reserves the right to make changes in the software without 
 * notification. NXP Semiconductors also make no representation or 
 * warranty that such application will be suitable for the specified 
 * use without further testing or modification. 
 *********************************************************************/

#include "..\..\..\swim\lpc_types.h"
#include "..\..\..\swim\lpc_swim.h"
#include "..\..\..\swim\lpc_swim_font.h"
#include "..\..\..\swim\lpc_swim.h"
#include "..\..\..\swim\lpc_rom8x16.h"
#include "..\..\..\swim\lpc_winfreesystem14x16.h"
#include "..\..\..\swim\lpc_x6x13.h"

#include "lpc_lcd_params.h"      /* LCD panel parameters */
#include "lpc2478.h"
#include "type.h"

extern DWORD timer0_counter; 
extern long dev_lcd ;

#define LPC2478_LCD_FRAME_BUF 0xA0000000
#define LCD_DISPLAY LCD_LTA057A347F_params16bit                 // IRD 2.0 + Toshiba LTA057A347F display 
#define FONT  font_x6x13 			// font_rom8x16 font_rom8x16 font_winfreesys14x16

/***********************************************************************
 *
 * Function: make_number_str
 *
 * Purpose: Convert a number to a positive decimal string
 *
 * Processing:
 *     Using successive division, compute the base10 value of a number
 *     into a string and return the string value to the caller.
 *
 * Parameters:
 *     str      : Where to place the generated strinjg
 *     iteration: Number to generate a string from
 *
 * Outputs: None
 *
 * Returns: Nothing
 *
 * Notes: None
 *
 **********************************************************************/
void make_number_str(CHAR *str,
                     INT_32 iteration)
{
    UNS_8 tmp[32];
    INT_32 dvv1, dvv2;
    INT_32 cnt = 0;

    /* Number 0 in string */
    str[cnt] = '0';

    /* Main loop */
    while (iteration > 0)
    {
        dvv1 = iteration / 10;
        dvv2 = dvv1 * 10;
        dvv1 = iteration - dvv2;

        tmp[cnt] = dvv1;
        iteration = iteration / 10;
        cnt++;
    }

    if (cnt > 0)
    {
        for (dvv1 = 0; dvv1 < cnt; dvv1++)
        {
            str[dvv1] = (CHAR) ('0' + tmp[cnt - (dvv1 + 1)]);
        }
    }
    
    str[cnt + 1] = '\0';
}


/**********************************************************************
 *
 * Function: lcd_colorbars
 *
 * Purpose: Draw colorbars on LCD panel
 *
 * Processing:
 *     See function.
 *
 * Parameters: None
 *
 * Outputs: None
 *
 * Returns: Nothing
 *
 * Notes: None
 *
 *********************************************************************/
void lcd_colorbars(void)
{
    SWIM_WINDOW_T win1;
    COLOR_T clr, *fblog;
    CHAR str[32];
	int idx;
    UNS_16 xgs, ygs, curx, cury, curym, xidx;

    /* Set LCD frame buffer address */
    fblog = (COLOR_T *)LPC2478_LCD_FRAME_BUF;

    /* Create a SWIM window */
    swim_window_open(&win1, LCD_DISPLAY.pixels_per_line,
        LCD_DISPLAY.lines_per_panel, fblog, 0, 0,
		(LCD_DISPLAY.pixels_per_line - 1), 
		(LCD_DISPLAY.lines_per_panel - 1),1, WHITE, BLACK, BLACK);

    /* Compute vertical size for 3 color bars */
    ygs = LCD_DISPLAY.lines_per_panel / 3;

    /* Draw Red bars */
    cury = 0;											// Start at 
    curx = 0;											// 0,0 position
    curym = ygs - 1;									// End of this color bar is 1/3 of panel 
    xgs = LCD_DISPLAY.pixels_per_line / RED_COLORS; 	// Divide pixels/line by # of red colors possible in this mode (32 shades red in RGB565)
    clr = BLACK;										// start with black then increase to full color across the line
    for (xidx = 0; xidx < RED_COLORS; xidx++)
    {
        swim_set_pen_color(&win1, clr);
        for (idx = 0; idx <= xgs; idx++)
        {
            swim_put_line(&win1, curx, cury, curx, curym);	// Draw line
            curx++;
        }
        clr = clr + 0x0400;		// increment color value for a gradient, Red=bits[14:10] in memory (RGB1:5:5:5)
    }

    /* Draw green bars */
    cury = cury + ygs;									// Next bar starts here (y offset) 
    curx = 0;											// 
    curym = cury + (ygs - 1);							// Ends here 
    xgs = LCD_DISPLAY.pixels_per_line / GREEN_COLORS;	// 
    clr = BLACK;										// gradient from black to greem maximum
    for (xidx = 0; xidx < GREEN_COLORS; xidx++)
    {
        swim_set_pen_color(&win1, clr);
        for (idx = 0; idx <= xgs; idx++)
        {
            swim_put_line(&win1, curx, cury, curx, curym);
            curx++;
        }
        clr = clr + 0x0020;		// incement green color value for a grdient, Green=bits[9:5] in memory (RGB1:5:5:5 mode)
    }

    /* Draw blue bars */
    cury = cury + ygs;
    curx = 0;
    curym = cury + (ygs - 1);
    xgs = LCD_DISPLAY.pixels_per_line / BLUE_COLORS;	// 
    clr = BLACK;
    for (xidx = 0; xidx < BLUE_COLORS; xidx++)
    {
        swim_set_pen_color(&win1, clr);
        for (idx = 0; idx <= xgs; idx++)
        {
            swim_put_line(&win1, curx, cury, curx, curym);
            curx++;
        }
        clr = clr + 0x0001;		// incement blue color value for a gradient, Blue=bits[4:0] in memory	(RGB1:5:5:5 mode)
    }

  /* select the font to use */
  swim_set_font(&win1, (FONT_T *)&FONT );

  /* set the pen color to use */
  swim_set_pen_color(&win1, WHITE);

  /* Add a title bar */
  swim_set_title(&win1, "NXP SWIM Graphics Library", BLACK);

  /* set the location to write text */
  swim_set_xy(&win1, 60, 160);

  /* set the pen color to use */
  swim_set_pen_color(&win1, LIGHTGRAY);

  /* put the timer tick on the panel */
  while(1){		
        swim_set_xy(&win1, 0, 0);
        swim_put_text(&win1, "Tick #");
        make_number_str(str, timer0_counter);
        swim_put_text(&win1, str);
        swim_put_text(&win1, "\n");
  }
	    
}
