
#include <LPC23xx.H>
#define SYS_FREQ  48

#if   SYS_FREQ == (72)
#define EMC_PERIOD          13.8  // 72MHz
#elif   SYS_FREQ == (57)
#define EMC_PERIOD          17.4  // 57.6MHz
#elif SYS_FREQ == (48)
#define EMC_PERIOD          20.8  // 48MHz
#elif SYS_FREQ == (36)
#define EMC_PERIOD          27.8  // 36MHz
#elif SYS_FREQ == (24)
#define EMC_PERIOD          41.7  // 24MHz
#else
#error Frequency not defined
#endif

#define SDRAM_REFRESH         7813	//max 64ms
#define SDRAM_TRP             20	 //
#define SDRAM_TRAS            45	
#define SDRAM_TAPR            1		  
//#define SDRAM_TDAL            2		//	tDAL=tWR+tRP
#define SDRAM_TDAL  			2+P2C(SDRAM_TRP)
#define SDRAM_TWR             2		
#define SDRAM_TRC             66 //samsung 65 micron 66
#define SDRAM_TRFC            66	
#define SDRAM_TXSR            1		
#define SDRAM_TRRD            15     
#define SDRAM_TMRD            2		

#define P2C(Period)     (((Period<EMC_PERIOD)?0:(unsigned int)((float)Period/EMC_PERIOD))+1)
#define SDRAM_BASE_ADDR *(volatile unsigned int*)0xA0000000 //DYCS0
#define SDRAM_CS0_BASE (0xA0000000)

#define FALSE (1 == 0)

#define TRUE  (1 == 1)


void GpioInit(void)
{
  // Set to inputs
  IODIR0  = \
  IODIR1  = \
  FIO0DIR = \
  FIO1DIR = \
  FIO2DIR = \
  FIO3DIR = \
  FIO4DIR = 0;

  // Enable Fast GPIO0,1
  SCS|=0x01;
 // SCS_bit.GPIOM = 1;

  // clear mask registers
  FIO0MASK =\
  FIO1MASK =\
  FIO2MASK =\
  FIO3MASK =\
  FIO4MASK = 0;

  // Reset all GPIO pins to default primary function
  PINSEL0 =\
  PINSEL1 =\
  PINSEL2 =\
  PINSEL3 =\
  PINSEL4 =\
  PINSEL5 =\
  PINSEL6 =\
  PINSEL7 =\
  PINSEL8 =\
  PINSEL9 =\
  PINSEL10= 0;
}

void EMC_Init (void)
{
  unsigned int i;
  unsigned short wtemp;

  //Pin Connect Block
  // Assign pins to EMC controller(SDRAM ) 

  PINSEL5&=0xF0FCFCC0;
  PINSEL5|=0x05010115;
  PINMODE5&=0xF0FCFCC0;
  PINMODE5|=0x0A02022A;
  //p2.29(DQMOUT1),28(DQMOUT0),24(CKEOUT0),20(DYCS0),18(CLKOUT0),17(RAS),16(CAS)
  //p2.16,17,18,20,24,28,29 mode=10
  //Pinmode=10 (Pin has neither pull-up nor pull-down resistor enabled.)
  
  PINSEL6  =  0x55555555;
  PINMODE6 =  0xAAAAAAAA;
  //p3.0-15=D0-15,mode=10
  
  PINSEL8 &= 0xC0000000;
  PINSEL8 |= 0x15555555;
  PINMODE8&= 0xC0000000;
  PINMODE8|= 0x2AAAAAAA;
  //p4.0-4.14=A0-14,mode=10
  
  PINSEL9 &= 0x3FF0C3FF;
  PINSEL9 |= 0x40051400;
  PINMODE9&= 0x3FF0C3FF;
  PINMODE9|= 0x800A2800;
  //p4.31(CS1),25(WE),24(OE),22(A22),21(A21),MODE=10  ,
  // p4.31(CS1),24(OE),22(A22),21(A21)	for nand flash
  
  // Init SDRAM controller
  // Enable EMC clock
  
  PCONP|=0x800;	    //enable EMC power
     
  EMC_CTRL=1;	  // enable EMC
  
 
  EMC_DYN_RD_CFG=1;//Configures the dynamic memory read strategy(Command delayed strategy)

  EMC_DYN_RASCAS0|=0x200;EMC_DYN_RASCAS0&=0xFFFFFEFF;//CAS latency=2

  EMC_DYN_RASCAS0|=0x3;	 // RAS latency(active to read/write delay)=3

  EMC_DYN_RP= P2C(SDRAM_TRP);
  EMC_DYN_RAS = P2C(SDRAM_TRAS);
  EMC_DYN_SREX = SDRAM_TXSR; 
  EMC_DYN_APR = SDRAM_TAPR; 
  EMC_DYN_DAL =SDRAM_TDAL ;
  EMC_DYN_WR = SDRAM_TWR;
  EMC_DYN_RC = P2C(SDRAM_TRC); 
  EMC_DYN_RFC = P2C(SDRAM_TRFC); 
  EMC_DYN_XSR = SDRAM_TXSR; 
  EMC_DYN_RRD = P2C(SDRAM_TRRD); 
  EMC_DYN_MRD = SDRAM_TMRD; 
  EMC_DYN_CFG0 = 0x0000680; 
  //16 bit external bus, 256 MB (16Mx16), 4 banks, row length = 13, column length = 9

  // JEDEC General SDRAM Initialization Sequence
  // DELAY to allow power and clocks to stabilize ~100 us
  // NOP
  EMC_DYN_CTRL = 0x0183;   
  //Issue SDRAM NOP (no operation) command ; CLKOUT runs continuously;All clock enables are driven HIGH continuously
  
  for(i = 200*30; i;i--);

  EMC_DYN_CTRL|=0x100; EMC_DYN_CTRL&=0xFFFFFF7F; //Issue SDRAM PALL (precharge all) command.
  EMC_DYN_RFSH = 1;  //Indicates 1X16 CCLKs between SDRAM refresh cycles.
  for(i= 128; i; --i); // > 128 clk
  EMC_DYN_RFSH = P2C(SDRAM_REFRESH) >> 4; // //Indicates ?? CCLKs between SDRAM refresh cycles.

  EMC_DYN_CTRL|=0x80; EMC_DYN_CTRL&=0xFFFFFEFF;   //Issue SDRAM MODE command.

  wtemp = *((volatile unsigned short *)(SDRAM_CS0_BASE | 0x00023000)); /* 8 burst, 2 CAS latency */

  EMC_DYN_CTRL = 0x0000;  //Issue SDRAM norm command ; CLKOUT stop;All clock enables low

  EMC_DYN_CFG0|=0x80000; //Buffer enabled for accesses to DCS0 chip

}

unsigned int  SDRAM_Test (void)
{
  unsigned int i;

  // 32 bits access
  for (i = 0; i < 0x2000000; i+=sizeof(unsigned int))
  {
    *(unsigned int *)((unsigned int )&SDRAM_BASE_ADDR+i) = i;
  }

  for (i = 0; i < 0x2000000; i+=sizeof(unsigned int ))
  {
    if (*(unsigned int *)((unsigned int )&SDRAM_BASE_ADDR+i) != i)
    {
      return(FALSE);
    }
  }

  // 16 bits access
  for (i = 0; i < 0x10000; i+=sizeof(unsigned short))
  {
    *(unsigned short*)((unsigned int)&SDRAM_BASE_ADDR+i) = i;
  }

  for (i = 0; i < 0x10000; i+=sizeof(unsigned short))
  {
    if (*(unsigned short*)((unsigned int)&SDRAM_BASE_ADDR+i) != i)
    {
      return(FALSE);
    }
  }

  // 8 bits access
  for ( i = 0; i < 0x100; i+=sizeof(unsigned char))
  {
    *(unsigned char*)((unsigned int)&SDRAM_BASE_ADDR+i) = i;
  }

  for (i = 0; i < 0x100; i+=sizeof(unsigned char))
  {
    if (*(unsigned char*)((unsigned int)&SDRAM_BASE_ADDR+i) != i)
    {
      return(FALSE);
    }
  }
  return(TRUE);
}	

int main()
{
  unsigned int Test;
  
  GpioInit();		 // Init GPIO
  
  EMC_Init();				 //Init EMC for SDRAM 
    
  Test=SDRAM_Test();
}
